/*
 * device.c
 *
 *  Created on: 25.04.2010
 *      Author: weisserd
 */



// hardware specific subroutines based on AVR Mega32


#define RFM_CLK_OUTPUT 0
#include <inttypes.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <string.h>
#include "hdef.h"
#include "device.h"
#include "rfm.h"
#include "rfm_config.h"
#include <avr/interrupt.h>



char message_buffer[64][6];
uint8_t message_rx_ptr;
uint8_t message_tx_ptr;
uint8_t rfm_buffer[20];
uint8_t cmd_cnt;
uint16_t ok_cnt;
uint8_t hex[16]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

struct message eth_message;


void _delayFourCycles(uint16_t __count)
{
      if ( __count == 0 )
            __asm__ __volatile__( "rjmp 1f\n 1:" );    // 2 cycles
      else
            __asm__ __volatile__ (
                                  "1: sbiw %0,1" "\n\t"
                        "brne 1b"                              // 4 cycles/loop
      : "=w" (__count)
      : "0" (__count)
                                 );
}



//-------------------------------------------------------------------------
// wait for t s
//-------------------------------------------------------------------------

void delay_sek(uint16_t t)
{
      uint16_t i,j;
      wdt_reset();
      for(j=0;j<t;j++){
            for(i=0;i<100;i++){
                  wdt_reset();
                  delay(10000);
            };
      };
}

//-------------------------------------------------------------------------
// wait for t ms
//-------------------------------------------------------------------------

void delay_ms(uint16_t t)
{
      uint16_t j;
      wdt_reset();
      for(j=0;j<t;j++){
            wdt_reset();
            delay(1000);
      };
}

//-------------------------------------------------------------------------
// init_spi()
//-------------------------------------------------------------------------
void init_spi(void)
{

      SPCR=(1<<SPE)|(1<<MSTR);
      SPSR=(1<<SPI2X);
}
//-------------------------------------------------------------------------
// calcCRC16r()
//-------------------------------------------------------------------------
uint16_t calcCRC16r( uint16_t c,uint16_t crc, uint16_t mask)  // reverser crc!!!!!!
{
uint8_t i;
  for(i=0;i<8;i++)
  {
    if((crc ^ c) & 1) { crc=(crc>>1)^mask; }
    else crc>>=1;
    c>>=1;
  };
 return(crc);
}
//-------------------------------------------------------------------------
// ror_buffer
//-------------------------------------------------------------------------
uint8_t ror_buffer(uint8_t cryin,uint8_t len,uint8_t *ptr)
{
	uint8_t *bptr;
	uint8_t c,c1,cry;
	bptr=ptr+(len-1);
	if(len==0)return(0);
	do{
		c=*bptr;
		cry=c&0x01;
		c>>=1;
		if(len>1){
			c1=*(bptr-1);
		}
		else{
			c1=cryin;
		};
		if(c1!=0)c|=0x80;
		*bptr=c;
		bptr--;
		len--;
	}while(len>0);
	return(cry);
}
//-------------------------------------------------------------------------
// rol_buffer
//-------------------------------------------------------------------------
uint8_t rol_buffer(uint8_t cryin,uint8_t len,uint8_t *ptr)
{

	uint8_t c,c1,cry;

	if(len==0)return(0);
	c=*ptr;
	if((c&0x80)!=0)cry=1;
	else cry=0;
	do{
		c=*ptr;
		c=(c<<1);
		if(len>1){
			c1=(*(ptr+1))&0x80;
		}
		else{
			if(cryin!=0)c1=0x80;
			else c1=0;
		};
		if(c1!=0)c|=0x01;
		*ptr=c;
		ptr++;
		len--;
	}while(len>0);
	return(cry);
}
//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
uint8_t code_message(struct message msg,uint16_t repeat)
{
	uint8_t t_buf[12],h_buf[12],i,cry;
	int8_t bc;
	uint8_t t;
	uint16_t crc=0;
	uint8_t j=0;

	t_buf[0]=0x7e;
	t_buf[1]=0x65;
	t_buf[3]=(uint8_t)(msg.adr>>16);
	t_buf[4]=(uint8_t)(msg.adr>>8);
	t_buf[5]=(uint8_t)(msg.adr);
	t_buf[6]=msg.cmd;
	if(msg.len==9){
		t_buf[2]=0x20;
		crc=0xbdb7;
		for(i=0;i<7;i++){
			crc=calcCRC16r(t_buf[i],crc,0x8408);
		};
		msg.crc=crc;
		t_buf[7]=(uint8_t)crc;
		t_buf[8]=(uint8_t)(crc>>8);
		t_buf[9]=0;
		t_buf[10]=0;
		t_buf[11]=0;
	}
	else if(msg.len==10){
		t_buf[2]=0x10;
		t_buf[7]=msg.data;
		crc=0xc11f;
		for(i=0;i<8;i++){
			crc=calcCRC16r(t_buf[i],crc,0x8408);
		};
		msg.crc=crc;
		t_buf[8]=(uint8_t)crc;
		t_buf[9]=(uint8_t)(crc>>8);
		t_buf[10]=0;
		t_buf[11]=0;
	};
	bc=0;
	for(t=0;t<12;t++)write_hex(t_buf[t]);
	write_nl();
	for(i=0;i<88;i++){
		cry=rol_buffer(0,12,t_buf);
		if(i<8)bc=0;					//erste 7e zulassen
		if(cry!=0){
			bc++;
			if(bc==5){
				rol_buffer(0,12,h_buf);					// zusätzliche 0 einschieben
				bc=0;
				j++;
			};
		}
		else{
			bc=0;
		};

		rol_buffer(cry,12,h_buf);


	};
	for(i=8;i>j;i--)rol_buffer(0,12,h_buf);
	for(t=0;t<12;t++)write_hex(h_buf[t]);
	write_nl();
	send_rfm_data(12,h_buf,repeat);
	return(0);
}
//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
uint8_t transmit_byte(uint8_t t)
{
uint16_t timeout=0x7fff;

      while(((PIN_NIRQ & NIRQ)!=0)&&((timeout--)>0));
      RFM_WRITE(t);
      if (timeout>0)return(0);
      else return (1);
}

//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
void send_rfm_data(uint8_t len,uint8_t *buf,uint16_t repeat)
{
uint16_t value=0;
uint8_t i,j,k,l;
uint8_t *bufstart;
uint16_t rep;


k=0;
 ok_cnt=0;


      RFM_SPI_16(RFM_TX_ON_PRE());
      delay_ms(6);
      RFM_READ_STATUS();
      RFM_WRITE(0x55);
      RFM_WRITE(0x55);
      RFM_SPI_16(RFM_TX_ON());

      transmit_byte(0xaa);
      transmit_byte(0xaa);
      for(rep=0;rep<repeat;rep++){

             bufstart=buf;
             for(l=0;l<len;l++){
                     i=*bufstart;
                    for (j=0;j<8;j++){
                           if((i&1)!=0){                 //Manchester 1
                                value=value<<2;
                                value+=2;
                            }
                            else{                         // Manchester 0
                                  value=value<<2;
                                  value+=1;
                           };
                           i=i>>1;
                    };
                    transmit_byte((uint8_t)(value>>8));
                    transmit_byte((uint8_t)(value&0xff));
                    wdt_reset();
                    bufstart++;
             };

      };
      transmit_byte(0x00);
      transmit_byte(0x00);
      delay_ms(5);
      RFM_SPI_16(RFM_TX_ON_PRE());
      delay_ms(5);





}
//---------------------------------------------------------------------
//
//---------------------------------------------------------------------
void write_hex(uint8_t hexval)
{
	TransmitByte_UART0(hex[hexval>>4]);
	TransmitByte_UART0(hex[hexval&0x0f]);
	TransmitByte_UART0(32);
}

void write_nl(void)
{
	TransmitByte_UART0(10);
	TransmitByte_UART0(13);
}
//---------------------------------------------------------------------
//
//---------------------------------------------------------------------
void write_string(char *cbuf)
{
	char c;
	while((c=*cbuf++)!=0){
		TransmitByte_UART0(c);
	};
	write_nl();
}
//---------------------------------------------------------------------
//
//---------------------------------------------------------------------
