// modul for receiving LON variables from Paradima MES unit

// WED 6.9.2009

#define USE_LON 1
#define RAW_DATA_JK 0
#define RAW_DATA 1
#define DECODE_BYTES_ETH 2				    /** nach 0x7e suchen und Manchester decodieren */
#define DECODE_BYTES_ETH_REV 3				/** nach 0x7e suchen und Manchester decodieren bits rückwärts */
#define BIT_MAX 0x90
#define BIT_MIN 0x10
#define LEN_2T 0x44

#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <avr/wdt.h>
#include <stdbool.h>
#include <avr/common.h>
#include <avr/sfr_defs.h>
#include "uart.h"

#include "lon.h"

#define BITLEN 0x170
#define MIDBIT 0xa0
#define SHORTMARK 0x80
#define PIN_LONRX PIND
#define LONRX 0x08
#define PORT_TESTPIN PORTD
#define TESTPIN 0x04;

#define MAX_VALUE 0xf0;
#define MIN_VALUE 0x01;


#define MAX_MSG 10

uint8_t rec_byte;
uint8_t *p;
volatile uint8_t bit_cnt;
volatile uint16_t buffer_cnt;
volatile uint16_t buffer_offset;


volatile uint8_t buf[1024];
volatile uint8_t islon;
volatile int8_t msg_cnt;
uint8_t b,new;

ISR(SIG_OVERFLOW0)
{
}

ISR(SIG_OVERFLOW1)
{
}

//
//
//
//
/*
ISR(SIG_INTERRUPT1)    // pin change on INT1 Bitwechsel Jochen
{
      b=TCNT0;
      TCNT0=0;
      if((b>BIT_MAX)||(b<BIT_MIN)){
    	  state=0;
      }
      else{


    		  if(state==0){									// wait for first bitchange 2T
				  if(b>LEN_2T)state=1;
				  if((PIN_LONRX & LONRX)!=0)state=0;		// level should be low
			  }
			  else if(state<=10){							// wait for 5 fullbit without bitchange 10 shortbits
				  if(b<LEN_2T)state++;
				  else state=1;								// bitchange found back to state 1
			  }
			  else if(state==11){							// now expecting bitchange (last bit in 7e is 0)
				  if(b<LEN_2T){
					  state=0;								// no bitchange -> back to search
				  };
				  state=20;									// now we found 7e sync finished
				  rbyte=0x7e;								// set shift value to 0
				  bit_cnt=8;								// clear bitcounter
				  buffer_cnt=0;								// clear buffercounter
				  bcnt=0;
				  lastbit=0;

			  }
			  else if(state==20){
				  if(b>LEN_2T){								// check for bitchange
					  if(lastbit!=0){
						  rbyte=(rbyte<<1);				 	// last bit was 1 new is 0
						  bcnt=0;
						  lastbit=0;
					  }
					  else {
						  rbyte=(rbyte<<1)|0x01;          	// last bit was 0 new is 1
						  bcnt++;
						  lastbit=1;
					  }
					  state=20;								// fullbit compleate
					  bit_cnt++;							// increase bit counter
				  }
				  else{
					  state=21;								// bit is halfbit, wait for next halfbit
				  };
			  }
			  else if(state==21){							// no bitchange
				  if(b<LEN_2T){								// next bit must be a halfbit
					  if(lastbit==0){
						  rbyte=(rbyte<<1);  // last bit was 0 new is 0
						  lastbit=0;
						  bcnt=0;
					  }
					  else {
						  rbyte=(rbyte<<1)|0x01;          	// last bit was 1 new is 1
						  lastbit=1;
						  bcnt++;
					  }
					  state=20;
					  bit_cnt++;
				  }
				  else{
					  state=0;							// bit is no halfbit -> Manchester violation
					 // state=20;
				  };
			  }else if(state==22){							// after 5 bit 1 skip one bit 0
				  if(b>LEN_2T){								// check for bitchange (next bit 0)
					  lastbit=0;
					  state=20;
				  }
				  else{

					 lastbit=1;
					 //state=11;
					 state=21;
				  }
				  bcnt=0;


			  }
    		  if(bcnt==5)state=22;

			  if(bit_cnt>7){								// wait for 8 bits
				  buf[buffer_cnt]=rbyte;					// save value into buffer
				  if(buffer_cnt<1020){
					  buffer_cnt++;
				  };
				  islon=1;									// set receiveflag
				  bit_cnt=0;								// clear bitcounter

			  };
	  };
}
*/

uint8_t manch_bits_0; // First  halfbit of manchester
uint8_t manch_bits_1; // Second halfbit of manchester
volatile uint8_t sync;

#define MANCH_SECOND_HALF	0x01

ISR(SIG_INTERRUPT1)
{
	b=TCNT0;
	TCNT0=0;

	// new contains the port status (0/1) _BEFORE_ the edge due to the comparison with 0
	new = ((PIN_LONRX & LONRX) != 0);
	PORTA=new;
	if (b < LEN_2T) {
		// SHORT bit: Shift into the current halfbit register the new bit
		if ((sync & MANCH_SECOND_HALF) == 0) {
			manch_bits_0 <<= 1;
			manch_bits_0 |=  new;
		}
		else {
			manch_bits_1 <<= 1;
			manch_bits_1 |=  new;

			process_manchester();
		}
		sync ^= MANCH_SECOND_HALF; // Toggle Manchester Halfbit
	}
	else {
		// LONG bit: Shift into both halfbit registers the new bit
		manch_bits_1 <<= 1;
		manch_bits_1 |=  new;
		process_manchester();
		manch_bits_0 <<= 1;
		manch_bits_0 |=  new;
		sync |= MANCH_SECOND_HALF; // After long bit, always Second halfbit expected.
	}
}


//short msg_cnt = -1;						defined in lon.h

void process_manchester() {
	// This routine is called only, when second manchesterbit has been received.
	PORTA|=0x02;
	// Check manchester validity
	if ((manch_bits_0 ^ manch_bits_1) == 0xff) {		// Manchestercode seems to be valid
		PORTA|=0x04;
		// The register manch_bits_1 contains the real data bits.

		// Waiting for Synchronization Byte 0x7e in raw bit stream
		if (manch_bits_1 == 0x7e) {
			PORTA|=0x10;
			msg_cnt++;
			if (msg_cnt == MAX_MSG) {
				msg_cnt--;
				islon=1;                            // set receive flag
			}
			else {
				PORTA|=0x20;
				p = &buf[buffer_offset];
				buffer_offset += 10;
				*p++ = 0x7e;
				buffer_cnt = 1;
				bit_cnt = 0;
			}
		}
		else if (buffer_cnt < 10) {
			PORTA|=0x40;
			// In case of being Byte synchron, a zero bit after five one bits needs to be skipped
			if ((manch_bits_1 & 0x3f) != 0x3e) {
				rec_byte >>= 1;
				if ((manch_bits_1 & 0x01) != 0)
					rec_byte |= 0x80;
				bit_cnt++;
				if (bit_cnt == 8) {
					PORTA|=0x80;
					*p++ = rec_byte;
					buffer_cnt++;
					bit_cnt = 0;
				}
			}
		}
	}
	else{
		PORTA|=0x08;

	}
}
