//--------------------------------------------------------------------------------
// V3.0     17.01.2009
//uart.c  Funktionen der Seriellen Schnittstellen
//
//
//
//--------------------------------------------------------------------------------


#include <avr/io.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "uart.h"

#define FOSC 3680000
#define BAUD 38400
#define MYUBRR 5  /*   38400 */

#define BUFFERSIZE 64

#ifdef UART0
  static uint8_t UART0_RxBuff[BUFFERSIZE];
  static uint8_t UART0_TxBuff[BUFFERSIZE];
  static volatile uint8_t UART0_RxHead;
  static volatile uint8_t UART0_RxTail;
  static volatile uint8_t UART0_TxHead;
  static volatile uint8_t UART0_TxTail;
#endif
#ifdef UART1
  static uint8_t UART1_RxBuff[BUFFERSIZE];
  static uint8_t UART1_TxBuff[BUFFERSIZE];
  static volatile uint8_t UART1_RxHead;
  static volatile uint8_t UART1_RxTail;
  static volatile uint8_t UART1_TxHead;
  static volatile uint8_t UART1_TxTail;
#endif
//! @brief interrupt service routine for RX interrupt \n
//!   One by one startup tests\n
//! @return result TEST_FAIL or TEST_OK
//!
//------------------------------------------------------------------------
//Empangsinterrupt
//------------------------------------------------------------------------
#ifdef UART0
ISR(SIG_UART_RECV)
{
uint8_t data;
uint8_t tmphead;

     data=UDR;
     tmphead=(UART0_RxHead+1)&(BUFFERSIZE-1);
     UART0_RxHead=tmphead;
     if(tmphead==UART0_RxTail)
     {

     };
     UART0_RxBuff[tmphead]=data;
}
#endif
#ifdef UART1
ISR(SIG_UART1_RECV)
{
uint8_t data;
uint8_t tmphead;

     data=UDR1;
     tmphead=(UART1_RxHead+1)&(BUFFERSIZE-1);
     UART1_RxHead=tmphead;
     if(tmphead==UART1_RxTail)
     {

     };
     UART1_RxBuff[tmphead]=data;
}
#endif

//------------------------------------------------------------------------
//Dataregister empty Interrupt
//------------------------------------------------------------------------
#ifdef UART0
ISR(SIG_UART_DATA)
{
uint8_t tmptail;
     if(UART0_TxHead!=UART0_TxTail)
     {
          tmptail=(UART0_TxTail+1)&(BUFFERSIZE-1);
          UART0_TxTail=tmptail;
          UDR=UART0_TxBuff[tmptail];
     }
     else{
          UCSRB&=~(1<<UDRIE);
     };
}
#endif
#ifdef UART1
ISR(SIG_UART1_DATA)
{
uint8_t tmptail;
     if(UART1_TxHead!=UART1_TxTail)
     {
          tmptail=(UART1_TxTail+1)&(BUFFERSIZE-1);
          UART1_TxTail=tmptail;
          UDR1=UART1_TxBuff[tmptail];
     }
     else{
          UCSR1B&=~(1<<UDRIE1);
     };
}
#endif

//------------------------------------------------------------------------
//init UART AT Mega64 Dual UART initialisieren
//------------------------------------------------------------------------
void InitUART(void)
{
#ifdef UART0

     UBRRH=(MYUBRR)>>8;
     UBRRL=0x05;
     UCSRB=((1<<RXCIE)|(1<<RXEN)|(1<<TXEN));
     UCSRC=((1<<URSEL)|(1<<USBS)|(3<<UCSZ0));   //URSEL wichtig bei Mega32
     UART0_RxTail=0;
     UART0_RxHead=0;
     UART0_TxHead=0;
     UART0_TxTail=0;
#endif
#ifdef UART1

     UBRR1H=(MYUBRR)>>8;
     UBRR1L=(uint8_t)MYUBRR;
     UCSR1B=((1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0));
     UCSR1C=((1<<USBS1)|(3<<UCSZ10));
     UART1_RxTail=0;
     UART1_RxHead=0;
     UART1_TxHead=0;
     UART1_TxTail=0;
#endif

}
//------------------------------------------------------------------------
//Empfangens Byte aus Puffer Holen
//------------------------------------------------------------------------
#ifdef UART0
uint8_t ReceiveByte_UART0(void)
{
     uint8_t tmptail;
     do{
     }while(UART0_RxHead==UART0_RxTail);
     tmptail=(UART0_RxTail+1)&(BUFFERSIZE-1);
     UART0_RxTail=tmptail;
     return(UART0_RxBuff[tmptail]);

}
#endif
#ifdef UART1
uint8_t ReceiveByte_UART1(void)
{
     uint8_t tmptail;
     do{
     }while(UART1_RxHead==UART1_RxTail);
     tmptail=(UART1_RxTail+1)&(BUFFERSIZE-1);
     UART1_RxTail=tmptail;
     return(UART1_RxBuff[tmptail]);

}
#endif

//------------------------------------------------------------------------
// Byte in Puffer schreiben
//------------------------------------------------------------------------
#ifdef UART0
void TransmitByte_UART0(uint8_t data)
{
uint8_t tmphead;
     tmphead=(UART0_TxHead+1)&(BUFFERSIZE-1);
     do{
     }while(tmphead==UART0_TxTail);
     UART0_TxBuff[tmphead]=data;
     UART0_TxHead=tmphead;
     UCSRB|=(1<<UDRIE);

}
#endif
#ifdef UART1
void TransmitByte_UART1(uint8_t data)
{
uint8_t tmphead;
     tmphead=(UART1_TxHead+1)&(BUFFERSIZE-1);
     do{
     }while(tmphead==UART1_TxTail);
     UART1_TxBuff[tmphead]=data;
     UART1_TxHead=tmphead;
     UCSR1B|=(1<<UDRIE1);

}
#endif
//------------------------------------------------------------------------
// Test auf Byte im Empfangspuffer
//------------------------------------------------------------------------
#ifdef UART0
uint8_t IsRxDataUART0(void)
{
     if(UART0_RxHead!=UART0_RxTail)return(1);
     else return(0);
}
#endif
#ifdef UART1
uint8_t IsRxDataUART1(void)
{
     if(UART1_RxHead!=UART1_RxTail)return(1);
     else return(0);
}
#endif
